function cnets = autoTrain( cnets, saveFolder, numParents, ...
    numOffspring, lrFactor, numGenerations, type)
%AUTOTRAIN Automatic training of cnns.
% INPUT cnets: Codat.CNN.cnn object OR
%              [Nx1] cell of Codat.CNN.cnn object OR
%               empty
%           If a Codat.CNN.cnn object or [Nx1] cell or cnn objects are
%           supplied then these are used as starting CNNs.
%           If cnet is empty than the newest generation will be loaded and
%           continued with the specified inputs.
%       saveFolder: string
%           Save output of each generation.
%       numParents: (Optional) int
%           Number of individuals that survive per generation.
%           (Default: 1)
%       numOffspring: (Optional) int
%           Number of offspring generated by *each* parent.
%           (Default: 3)
%       lrFactor: (Optional) double
%           The learning rate of each parent is evolved by numOffspring
%           powers of the lrFactor. The powers are symetrically centered
%           around zero.
%           (Default: 10)
%       numGenerations: (Optional) int
%           Total number of evolutionary iterations.
%           (Default: 5)
%       type: (Optional) string
%           see Codat.CNN.Cluster.train
%           (Default: 'membraneWeighted')
%           
% Author: Benedikt Staffler <benedikt.staffler@brain.mpg.de>

%prepare paths
saveFolder = Util.addFilesep(saveFolder);
tmpFolder = [saveFolder 'tmp' filesep];
if ~exist(tmpFolder,'dir')
    mkdir(tmpFolder);
    if length(dir(tmpFolder)) > 2
        warning('tmpFolder is not empty. Possible deletion of data.');
    end
end

%prepare cnets
if isa(cnets, 'Codat.CNN.cnn')
    cnets = {cnets};
    numStart = 0;
elseif iscell(cnets)
    numStart = 0;
elseif isempty(cnets)
    %get newest generation
    list = dir(saveFolder);
    list([list.isdir]) =  [];
    list(arrayfun( ...
        @(x)isempty(regexp(x.name, 'Gen\d*_P\d.mat', 'once')), list)) = [];
    [~,newest] = max([list.datenum]);
    m = load([saveFolder list(newest).name]);
    cnets = {m.cnet};
    delete([tmpFolder '*.mat']); %delete old generations
    numStart = length(list);
else
    error('Wrong input type for cnets.');
end

if ~exist('numParents','var') || isempty(numParents)
    numParents = 1;
end
if ~exist('numOffspring','var') || isempty(numOffspring)
    numOffspring = 3;
end
if ~exist('lrFactor','var') || isempty(lrFactor)
    lrFactor = 10;
end
if ~exist('numGenerations','var') || isempty(numGenerations)
    numGenerations = 5;
end
if ~exist('type','var') || isempty(type)
    type = 'membraneWeighted';
end

%folders for saving generations and current iteration
saveFolder = Util.addFilesep(saveFolder);
if ~exist(saveFolder,'dir')
    mkdir(saveFolder);
end


fprintf('[%s] Starting training.\n', datestr(now));
lrFactor = lrFactor.^((1:numOffspring) - (numOffspring+1)/2);
cluster = getCluster('gpu');
if numStart == 0
    save([saveFolder 'Gen0_P.mat'], 'cnets');
end

for i = (numStart + 1):(numStart + numGenerations)
    fprintf('[%s] Training generation %d.\n', datestr(now), i);
    cnets = createOffspring(cnets, lrFactor);
    
    %train classifier
    jobs = Codat.CNN.Cluster.train(type, cnets, tmpFolder, ...
        ['Gen', num2str(i)], cluster);
    
    fprintf('[%s] Waiting for jobs %s.\n', datestr(now), ...
        strrep(num2str([jobs.Id]), '  ', ', '));
    
    %wait till output
    pause(100*60*60); %100 h - current maximum at gaba
    
    fprintf('[%s] Evaluating generation %d.\n', datestr(now), i);
    %load trained cnets
    list = dir(tmpFolder);
    list([list.isdir]) =  [];
    
    loss = zeros(length(cnets), 1);
    m = cell(length(cnets),1);
    cnetsNew = cell(length(cnets),1);
    for j = 1:length(cnets)
        idx = strfind({list.name}, ['Gen', num2str(i), '_' num2str(j)]);
        idx = find(~cellfun(@isempty,idx));
        if ~isempty(idx)
            [~,newest] = max([list(idx).datenum]);
            m{j} = load([tmpFolder list(idx(newest)).name]);
            m{j}.parent = ceil(j/numOffspring); %keep track of parent cnet
            loss(j) = m{j}.validationLoss(end);
            cnetsNew{j} = m{j}.cnet;
        else
            fprintf(['[%s] Could not find any trained cnets for ' ...
                'Generation%d_%d which will thus be ignored.'], ...
                datestr(now), i, j)
            loss(j) = Inf;
        end
    end
    
    if all(isinf(loss))
        error('[%s] No cnets for found for generation %d.\n', ...
            datestr(now), i);
    end
    
    [~, idx] = sort(loss, 'ascend');
    
    %save best cnet
    for j = 1:numParents
        s = m{idx(j)}; %#ok<NASGU>
        save([saveFolder 'Gen' num2str(i) '_P' num2str(j) '.mat'], ...
            '-struct', 's');
    end
    
%     %remove old training files
%     delete([tmpFolder '*.mat']);
    
    cnets = cnetsNew(idx(1:numParents));
    fprintf('[%s] Finished iteration %d/%d.\n',datestr(now), i, ...
        numGenerations);
end
fprintf('[%s] Finished training.\n', datestr(now));

end

function cnets = createOffspring(cnets, lrFactor)
cnets = repmat(cnets, 1, length(lrFactor));
for i = 1:size(cnets, 1)
    baseLR = cnets{i,1}.optimizer.learningRate;
    for j = 1:size(cnets,2)
        cnets{i,j}.optimizer.learningRate = baseLR*lrFactor(j);
    end
end
cnets = reshape(cnets, numel(cnets), 1);
end

